/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef PERCEPTION_IMPL_TYPE_CAMERAPERCEPTION2FUSION_H
#define PERCEPTION_IMPL_TYPE_CAMERAPERCEPTION2FUSION_H

#include <cfloat>
#include <cmath>
#include "impl_type_point3d.h"
#include "impl_type_double.h"
#include "impl_type_uint8.h"
#include "impl_type_uint32.h"
#include "impl_type_uint64.h"
#include "impl_type_vectorfloat.h"

namespace perception {
struct CameraPerception2Fusion {
    ::Point3D position;
    ::Double theta{ 0.0 };
    ::Double length{ 0.0 };
    ::Double width{ 0.0 };
    ::Double height{ 0.0 };
    ::Point3D velocity;
    ::UInt8 type{ 0U };
    ::UInt8 subtype{ 0U };
    ::Double confidence{ 0.0 };
    ::UInt32 ID{ 0U };
    ::Double timestamp{ 0.0 };
    ::UInt64 measurement_time{ 0U };
    ::Point3D acceleration;
    ::VectorFloat position_covariance;
    ::VectorFloat velocity_covariance;
    ::VectorFloat acceleration_covariance;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(position);
        fun(theta);
        fun(length);
        fun(width);
        fun(height);
        fun(velocity);
        fun(type);
        fun(subtype);
        fun(confidence);
        fun(ID);
        fun(timestamp);
        fun(measurement_time);
        fun(acceleration);
        fun(position_covariance);
        fun(velocity_covariance);
        fun(acceleration_covariance);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(position);
        fun(theta);
        fun(length);
        fun(width);
        fun(height);
        fun(velocity);
        fun(type);
        fun(subtype);
        fun(confidence);
        fun(ID);
        fun(timestamp);
        fun(measurement_time);
        fun(acceleration);
        fun(position_covariance);
        fun(velocity_covariance);
        fun(acceleration_covariance);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("position", position);
        fun("theta", theta);
        fun("length", length);
        fun("width", width);
        fun("height", height);
        fun("velocity", velocity);
        fun("type", type);
        fun("subtype", subtype);
        fun("confidence", confidence);
        fun("ID", ID);
        fun("timestamp", timestamp);
        fun("measurement_time", measurement_time);
        fun("acceleration", acceleration);
        fun("position_covariance", position_covariance);
        fun("velocity_covariance", velocity_covariance);
        fun("acceleration_covariance", acceleration_covariance);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("position", position);
        fun("theta", theta);
        fun("length", length);
        fun("width", width);
        fun("height", height);
        fun("velocity", velocity);
        fun("type", type);
        fun("subtype", subtype);
        fun("confidence", confidence);
        fun("ID", ID);
        fun("timestamp", timestamp);
        fun("measurement_time", measurement_time);
        fun("acceleration", acceleration);
        fun("position_covariance", position_covariance);
        fun("velocity_covariance", velocity_covariance);
        fun("acceleration_covariance", acceleration_covariance);
    }

    friend bool operator==(const ::perception::CameraPerception2Fusion& lhs, const ::perception::CameraPerception2Fusion& rhs) noexcept
    {
        return (lhs.position == rhs.position) && (fabs(static_cast<double>(lhs.theta - rhs.theta)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.length - rhs.length)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.width - rhs.width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.height - rhs.height)) < DBL_EPSILON) && (lhs.velocity == rhs.velocity) && (lhs.type == rhs.type) && (lhs.subtype == rhs.subtype) && (fabs(static_cast<double>(lhs.confidence - rhs.confidence)) < DBL_EPSILON) && (lhs.ID == rhs.ID) && (fabs(static_cast<double>(lhs.timestamp - rhs.timestamp)) < DBL_EPSILON) && (lhs.measurement_time == rhs.measurement_time) && (lhs.acceleration == rhs.acceleration) && (lhs.position_covariance == rhs.position_covariance) && (lhs.velocity_covariance == rhs.velocity_covariance) && (lhs.acceleration_covariance == rhs.acceleration_covariance);
    }
};
} // namespace perception


#endif // PERCEPTION_IMPL_TYPE_CAMERAPERCEPTION2FUSION_H
